  '********************************************************************
  '** Program Name:  RJCHKDUP - Check for duplicate files.           **
  '**       Author:  R.J. (Bob) Ross                                 **
  '**                SysOp - RJ's Byteline BBS - FidoNet 1:134/75    **
  '**                                                                **
  '** Date Started:  March 13th, 1995                                **
  '**      Version:  1.00 - completed March 25th, 1995               **
  '**                Last compiled March 26th, 1995                  **
  '**                                                                **
  '**      Purpose:  To produce a listing of duplicate files which   **
  '**                can be used with program RJKILL to delete       **
  '**                duplicates and update appropriate Files.bbs     **
  '**                entries.                                        **
  '**                                                                **
  '**     Language:  Microsoft QuickBASIC 4.5                        **
  '**    Libraries:  Crescent Software Inc - QuickPak Professional   **
  '**                                                                **
  '********************************************************************
  
    DEFINT A-Z
    DECLARE FUNCTION Exist% (FileName$)
    DECLARE FUNCTION FindLast% (BYVAL Address, Size)
    DECLARE FUNCTION CurrentDate$ ()
    DECLARE FUNCTION DiskRoom& (Drive$)
    DECLARE SUB Help ()
    DECLARE SUB ParseCmdLine (Switch$, Work$, ReturnCode%)
    DECLARE FUNCTION QInstrB% (StartPos%, Source$, Search$)
    DECLARE FUNCTION QPTrim$ (Work$)
  
    CONST False = 0
    CONST True = NOT False
    CONST Version = "RJCHKDUP Version 1.00"
    Dupes% = False
    DelTemp% = True
    Dt$ = CurrentDate$
    Count& = 0
    CLS
  
    IF COMMAND$ = "" THEN
        CALL Help
        END
    END IF
  
    PRINT
    PRINT Dt$; TIME$; " RJUT "; "Begin, RJCHKDUP Version 1.00 "; "String Memory = "; FRE("X")
    PRINT Dt$; TIME$; " RJUT Disk Room ="; DiskRoom&(Drive$)
  
  'Look for a Config file (-C)
    CALL ParseCmdLine("-C", ConfigFile$, ConfigFile%)
    IF ConfigFile% THEN
        IF Exist%(ConfigFile$) THEN
            Cfg% = FREEFILE
            OPEN ConfigFile$ FOR INPUT AS #Cfg%
            IF LOF(Cfg%) = 0 THEN
                CLOSE Cfg%
                PRINT Dt$; TIME$; " RJUT is empty."
                PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
                END
            END IF
            CLOSE 2
        ELSE
            BEEP
            PRINT "Config file "; ConfigFile$; " NOT found! "
            END
        END IF
      'No command line option given - display help screen.
    END IF
  
  'Duplicate File Listing - O
    CALL ParseCmdLine("-O", OutPutFile$, OutPutFile%)
  
    IF OutPutFile% AND ConfigFile% THEN
      'do nothing
    ELSEIF NOT ConfigFile% THEN
        PRINT Dt$; TIME$; " RJUT Configuration file not called on Cmd Line (-C)"
        PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        END
    ELSEIF NOT OutPutFile% THEN
        PRINT Dt$; TIME$; " RJUT No output file given on Cmd Line (-O)"
        PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        END
    END IF
   'Keep temp file $$Mast.Lst (-KTF)
    IF INSTR(COMMAND$, "-KTF") THEN
        DelTemp% = False
    END IF
  
    Cfg% = FREEFILE
    OPEN ConfigFile$ FOR INPUT AS #Cfg%
    MasterLst$ = "$$mast.lst"
    ML% = FREEFILE
    OPEN MasterLst$ FOR OUTPUT AS ML%
  
    DO UNTIL EOF(Cfg%)
        DO UNTIL EOF(Cfg%)
            LINE INPUT #Cfg%, ConfigRec$
            ConfigRec$ = UCASE$(LTRIM$(ConfigRec$))
            IF LEFT$(ConfigRec$, 12) = "AREAFILESBBS" THEN
                FilesBBS$ = QPTrim$(MID$(ConfigRec$, 13))
            ELSE
                EXIT DO
            END IF
            IF Exist%(FilesBBS$) THEN
                BBS% = FREEFILE
                OPEN FilesBBS$ FOR INPUT AS #BBS%
                P% = QInstrB%(-1, FilesBBS$, "\")
                IF P% THEN Path$ = LEFT$(FilesBBS$, P%)
                GOSUB GetFilenames
            ELSE
                BEEP
                PRINT "File "; FilesBBS$; " not found!"
                PRINT Dt$; TIME$; " RJUT File "; FilesBBS$; " not found!"
                EXIT DO
            END IF
        LOOP
    LOOP
    CLOSE Cfg%
    CLOSE ML%
  
    TempFile$ = "$$Temp."
    MergeFile$ = "$$Merg."
  
    PRINT Dt$; TIME$; " RJUT Number of Lines in "; MasterLst$; Count&
  
  'Find out how many temp files to make
    NumberOfPasses% = Count& \ 1000
    Remainder% = Count& MOD 1000
    IF Remainder% THEN
        NumberOfPasses% = NumberOfPasses% + 1
    END IF
  
  'Calculate the first merge
    NumberOfMerges% = NumberOfPasses% \ 2
    Odd% = NumberOfPasses% MOD 2
  
    CLOSE 1
    OPEN MasterLst$ FOR INPUT AS #1
  
    FileSize& = LOF(1)
    PRINT Dt$; TIME$; " RJUT Size of "; MasterLst$; " ="; FileSize&
  
  'Make sure there is enough disk space to work with
    IF (FileSize& * 2) + 2000 >= DiskRoom&(Drive$) THEN
        PRINT Dt$; TIME$; " RJUT Insufficient disk space to perform sort."
        PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        CLOSE
        KILL MasterLst$
    END IF
  
    Last% = 1000
    REDIM FlName$(1 TO 1000)
  
    IF NumberOfPasses% = 1 THEN
        GOSUB FillFlName
        GOSUB LastFullArray
        GOSUB Sort
        CLOSE 1
        OPEN MasterLst$ FOR OUTPUT AS #1
        FOR X% = 1 TO Last%
            PRINT #1, FlName$(X%)
        NEXT
        CLOSE 1
        GOSUB EmptyIt
        PRINT Dt$; TIME$; " RJUT Finished Sort of "; MasterLst$; " String Memory ="; FRE("X")
        GOSUB MakeDupeList
        IF Dupes% THEN
            PRINT Dt$; TIME$; " RJUT Finished "; OutPutFile$
            PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        ELSE
            PRINT Dt$; TIME$; " RJUT No Dupes Found!"
            PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
            KILL OutPutFile$
        END IF
        IF DelTemp% THEN
            KILL MasterLst$
            ELSE
            PRINT Dt$; TIME$; " RJUT File: "; MasterLst$; " kept on disk."
        END IF
        END
    END IF
  
    PRINT Dt$; TIME$; " RJUT Passes :"; NumberOfPasses%; "String Memory ="; FRE("X")
    FOR Temp% = 1 TO NumberOfPasses%
        OutFile$ = TempFile$ + LTRIM$(STR$(Temp%))
        GOSUB Initialize
        GOSUB FillFlName
        GOSUB LastFullArray
        GOSUB Sort
        OPEN OutFile$ FOR OUTPUT AS #2
        FOR X% = 1 TO Last%
            PRINT #2, FlName$(X%)
        NEXT
        CLOSE 2
    NEXT
    CLOSE 1
    GOSUB EmptyIt
  
    IF NumberOfPasses% = 2 THEN
        File1$ = TempFile$ + "1"
        File2$ = TempFile$ + "2"
        OutFile$ = MergeFile$ + "1"
        PRINT Dt$; TIME$; " RJUT Merging...."
        GOSUB MergeFiles
        KILL File1$: KILL File2$
        KILL MasterLst$
        NAME OutFile$ AS MasterLst$
        GOSUB EmptyIt
        PRINT Dt$; TIME$; " RJUT Finished Sort of "; MasterLst$; " String Memory ="; FRE("X")
        GOSUB MakeDupeList
        IF Dupes% THEN
            PRINT Dt$; TIME$; " RJUT Finished "; OutPutFile$
            PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        ELSE
            PRINT Dt$; TIME$; " RJUT No Dupes Found!"
            PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
            KILL OutPutFile$
        END IF
        IF DelTemp% THEN
            KILL MasterLst$
            ELSE
            PRINT Dt$; TIME$; " RJUT File: "; MasterLst$; " kept on disk."
        END IF
        END
    END IF
  
Merge:
    PRINT Dt$; TIME$; " RJUT Merging:"; NumberOfMerges%
    a% = 1
    FOR X% = 1 TO NumberOfMerges%
        OutFile$ = MergeFile$ + LTRIM$(STR$(X%))
        File1$ = TempFile$ + LTRIM$(STR$(a%))
        File2$ = TempFile$ + LTRIM$(STR$(a% + 1))
        GOSUB MergeFiles
        KILL File1$
        KILL File2$
        a% = a% + 2
        NAME OutFile$ AS TempFile$ + LTRIM$(STR$(X%))
    NEXT
  
    IF Odd% THEN
        NAME TempFile$ + LTRIM$(STR$(NumberOfPasses%)) AS TempFile$ + LTRIM$(STR$(NumberOfMerges% + 1))
        PRINT Dt$; TIME$; " RJUT Merging:"; NumberOfMerges% + 1
    END IF
  
    NumberOfPasses% = NumberOfMerges% + Odd%
    Odd% = 0
    NumberOfMerges% = NumberOfPasses% \ 2
    Odd% = NumberOfPasses% MOD 2
    IF NumberOfMerges% THEN
        GOTO Merge
    END IF
    KILL MasterLst$
    NAME TempFile$ + "1" AS MasterLst$
    PRINT Dt$; TIME$; " RJUT Finished Sort of "; MasterLst$; " String Memory ="; FRE("X")
    GOSUB MakeDupeList
    IF Dupes% THEN
        PRINT Dt$; TIME$; " RJUT Finished "; OutPutFile$
        PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
    ELSE
        PRINT Dt$; TIME$; " RJUT No Dupes Found!"
        PRINT Dt$; TIME$; " RJUT End RJCHKDUP"
        KILL OutPutFile$
    END IF
    IF DelTemp% THEN
        KILL MasterLst$
        ELSE
        PRINT Dt$; TIME$; " RJUT File: "; MasterLst$; " kept on disk."
    END IF
    PRINT
    END
  
GetFilenames:
    DO UNTIL EOF(BBS%)
        LINE INPUT #BBS%, Record$
        P% = INSTR(Record$, ".")
        IF P% THEN
            IF P% < 10 AND LEFT$(Record$, 1) <> " " THEN
                P% = INSTR(Record$, " ")
                IF P% THEN
                    Ctr% = Ctr% + 1
                    FileName$ = LEFT$(Record$, P% - 1)
                    PRINT #ML%, FileName$; ","; Path$
                    F% = LEN(FileName$)
                    Pa% = LEN(Path$)
                    LineLength% = F% + Pa%
                    IF LineLength% >= LongestLine% THEN
                        LongestLine% = LineLength%
                        LineLength% = LongestLine%
                    END IF
                    Count& = Count& + 1
                END IF
            END IF
        END IF
    LOOP
    CLOSE BBS%
    RETURN
  
EmptyIt:
    ERASE FlName$
    RETURN
  
Initialize:
    REDIM FlName$(1 TO 1000)
    RETURN
  
FillFlName:
    FOR X% = 1 TO Last%
        IF EOF(1) THEN EXIT FOR
        LINE INPUT #1, Record$
        FlName$(X%) = Record$
    NEXT
    RETURN
  
Sort:
    CALL SortStr(BYVAL VARPTR(FlName$(1)), Last%, 0)
    RETURN
  
LastFullArray:
    Last% = FindLast%(BYVAL VARPTR(FlName$(1000)), 1000)
    RETURN
  
MergeFiles:
  ' code by Matt Hart
    OPEN File1$ FOR INPUT AS 1
    OPEN File2$ FOR INPUT AS 2
    OPEN OutFile$ FOR OUTPUT AS 3
    DO
        IF LEN(S1$) = 0 AND NOT EOF(1) THEN LINE INPUT #1, S1$
        IF LEN(S2$) = 0 AND NOT EOF(2) THEN LINE INPUT #2, S2$
        IF LEN(S1$) > 0 AND LEN(S2$) > 0 THEN
            IF S1$ < S2$ THEN
                PRINT #3, S1$: S1$ = ""
            ELSE
                PRINT #3, S2$: S2$ = ""
            END IF
        ELSEIF LEN(S1$) THEN
            PRINT #3, S1$: S1$ = ""
        ELSEIF LEN(S2$) THEN
            PRINT #3, S2$: S2$ = ""
        ELSEIF EOF(1) AND EOF(2) THEN
            EXIT DO
        END IF
    LOOP
    CLOSE 1, 2, 3
    RETURN
  
MakeDupeList:
    OPEN MasterLst$ FOR INPUT AS #1
    OPEN OutPutFile$ FOR OUTPUT AS #2
    PRINT Dt$; TIME$; " RJUT Making: "; OutPutFile$
    DO UNTIL EOF(1)
        INPUT #1, Var2$, Seq2$
        P% = INSTR(Var2$, ".")
        IF LEFT$(Var2$, P%) = LEFT$(Var1$, P%) THEN
            IF LEN(Var1$) THEN
              ' PRINT Var1$, Seq1$
                Path$ = Seq1$
                FileName$ = Var1$
                GOSUB GetDescription
                PRINT #2, RTRIM$(Seq1$); Var1$;
                PRINT #2, TAB(LongestLine% + 1); LTRIM$(Description$)
                PRINT Dt$; TIME$; " RJUT Dupe: "; RTRIM$(Seq1$); Var1$
                Dupes% = True
            END IF
            IF LEN(Var2$) THEN
                Path$ = Seq2$
                FileName$ = Var2$
                GOSUB GetDescription
                PRINT #2, RTRIM$(Seq2$); RTRIM$(Var2$);
                PRINT #2, TAB(LongestLine% + 1); LTRIM$(Description$)
                PRINT Dt$; TIME$; " RJUT Dupe: "; RTRIM$(Seq2$); Var2$
            END IF
            Var1$ = ""
            Seq1$ = ""
            Var3$ = Var2$
            Seq3$ = Seq2$
        ELSE
            Var1$ = Var2$
            Seq1$ = Seq2$
            IF LEFT$(Var1$, P%) = LEFT$(Var3$, P%) THEN
                Path$ = Seq1$
                FileName$ = Var1$
                GOSUB GetDescription
                PRINT #2, RTRIM$(Seq1$); Var1$;
                PRINT #2, TAB(LongestLine% + 1); LTRIM$(Description$)
                PRINT Dt$; TIME$; " RJUT Dupe: "; RTRIM$(Seq1$); Var1$
                Dupes% = True
                Var1$ = "": Seq1$ = ""
            END IF
        END IF
    LOOP
    CLOSE
    RETURN
  
GetDescription:
    IF Exist%(Path$ + "Files.bbs") THEN
        OPEN Path$ + "Files.bbs" FOR INPUT AS #3
        DO UNTIL EOF(3)
            LINE INPUT #3, Record$
            IF LEFT$(Record$, LEN(FileName$)) = FileName$ THEN
                P% = INSTR(Record$, " ")
                IF P% THEN
                    Description$ = MID$(Record$, P%)
                    EXIT DO
                END IF
            END IF
        LOOP
        CLOSE 3
    ELSE
        Description$ = Path$ + "Files.bbs not found!"
    END IF
    RETURN

    FUNCTION CurrentDate$ STATIC
        Dt$ = DATE$
        Mnth$ = LEFT$(Dt$, 2)
        SELECT CASE Mnth$
        CASE IS = "01"
            Month$ = "Jan"
        CASE IS = "02"
            Month$ = "Feb"
        CASE IS = "03"
            Month$ = "Mar"
        CASE IS = "04"
            Month$ = "Apr"
        CASE IS = "05"
            Month$ = "May"
        CASE IS = "06"
            Month$ = "Jun"
        CASE IS = "07"
            Month$ = "Jul"
        CASE IS = "08"
            Month$ = "Aug"
        CASE IS = "09"
            Month$ = "Sep"
        CASE IS = "10"
            Month$ = "Oct"
        CASE IS = "11"
            Month$ = "Nov"
        CASE IS = "12"
            Month$ = "Dec"
        END SELECT
        CurrentDate$ = " " + MID$(Dt$, 4, 2) + " " + Month$ + " "
      
    END FUNCTION

    SUB Help
        CLS
        Temp$ = "RJ's Check Duplicate Files Program - " + Version
        Temp2$ = "by R.J. (Bob) Ross"
        Temp3$ = "SysOp - RJ's Byteline BBS - FidoNet 1:134/75"
        PRINT TAB(40 - LEN(Temp$) \ 2); Temp$
        PRINT TAB(40 - LEN(Temp2$) \ 2); Temp2$
        PRINT TAB(40 - LEN(Temp3$) \ 2); Temp3$
        PRINT STRING$(78, 196)
        PRINT "Purpose:  To read all the Files.bbs files listed in the AreaFilesBBS"
        PRINT "          portion of the configuration file and check for duplicated"
        PRINT "          files.  Produces a text listing of duplicates which can"
        PRINT "          be edited and used for program RJKILL to delete files and"
        PRINT "          update appropriate Files.bbs entries."
        PRINT STRING$(78, 196)
        PRINT "Switches:"
        PRINT
        PRINT "          -Cdrive:\path\Sample.cfg"
        PRINT "          -Odrive:\path\KilList.txt"
        PRINT "          -KTF [optional] - keeps file $$Mast.Lst"
        PRINT
        PRINT
    END SUB

    SUB ParseCmdLine (Switch$, Work$, ReturnCode%)
      
        CmdLine$ = UCASE$(COMMAND$)
        ReturnCode% = False
      
        P% = INSTR(CmdLine$, Switch$)
        IF P% THEN
            Temp$ = MID$(CmdLine$, P% + 2)
            P% = INSTR(Temp$, " ")
            IF P% THEN
                Work$ = LEFT$(Temp$, P% - 1)
            ELSE
                Work$ = Temp$
            END IF
            ReturnCode% = True
        END IF
      
    END SUB

